﻿<#
.SYNOPSIS
  Queries Lync Server 2010/2013 pools and Skype for Business 2015 pools and retrieves what Cumulative Update is installed on each server in each pool.

.DESCRIPTION
  Queries Lync Server 2010/2013 pools and Skype for Business 2015 pools and retrieves what Cumulative Update is installed on each server in each pool.

.NOTES
  Version               : 3.9 - See changelog at https://www.ucunleashed.com/2642
  Wish list             : Better error trapping
  Rights Required       : CSAdministrator and local admin rights on the target computers
  Sched Task Required   : No
  Lync/Skype4B Version  : 2010 and 2013, and Skype for Business Server 2015
  Author(s)             : Dave Howe
                        : Pat Richard
  Email/Blog/Twitter    : daveh@microsoft.com
                          pat@innervation.com   https://www.ucunleashed.com @patrichard
  Donations             : https://www.paypal.me/PatRichard
  Dedicated Post        : https://www.ucunleashed.com/2641
  Disclaimer            : You running this script means you won't blame author(s) if this breaks your stuff. This script is
                          provided AS IS without warranty of any kind. Author(s) disclaim all implied warranties including,
                          without limitation, any implied warranties of merchantability or of fitness for a particular
                          purpose. The entire risk arising out of the use or performance of the sample scripts and
                          documentation remains with you. In no event shall author(s) be liable for any damages whatsoever
                          (including, without limitation, damages for loss of business profits, business interruption,
                          loss of business information, or other pecuniary loss) arising out of the use of or inability
                          to use the script or documentation. Neither this script, nor any part of it other than those
                          parts that are explicitly copied from others, may be republished without author(s) express written 
                          permission.
  Acknowledgements      :
  Assumptions           : ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations           : PSRemoting must be enabled on the servers that are queried. It's enabled by default on Windows Server 2012
                          and later. For information on enabling PSRemoting, see http://technet.microsoft.com/en-us/library/hh849694.aspx
  Known issues          : Edge servers that aren't domain joined will show as "PSRemoting Failure".

.EXAMPLE
  .\Get-CsUpdateVersion.ps1

  Description
  -----------
  Runs the script and queries all pools in the environment.

.EXAMPLE
  .\Get-CsUpdateVersion.ps1 -PoolFqdn pool.contoso.com

  Description
  -----------
  Runs the script and queries just the pool specified.
#>
#Requires -Version 3.0

[CmdletBinding(SupportsShouldProcess)]
param (
  # Specifies a single pool to query
  [parameter(ValueFromPipeline, ValueFromPipelineByPropertyName)]
  [string] $PoolFqdn,

  # Specifies the Time To Live for connection attempts to servers
  [parameter()]
  [ValidateNotNullOrEmpty()]
  [int] $TTL = 10,
  
  # Defines the path to the parent folder of the logs folder.
  [parameter()]
  [ValidateNotNullOrEmpty()]
  [string] $TargetFolder = "$env:SystemDrive\_Install"
)
[string] $ScriptVersion = '3.9'
[string] $LogPath = "$TargetFolder\logs\$env:ComputerName" + " {0:yyyy-MM-dd hh-mmtt}.log" -f (Get-Date)
[string] $LogDivider = '+------------------------------+'

#region functions
function Get-UpdateInfo {
	<#
	.SYNOPSIS
	  Queries an online XML source for version information to determine if a new version of the script is available.

	.DESCRIPTION
	  Queries an online XML source for version information to determine if a new version of the script is available.

	.NOTES
	  Version               : 1.2 - See changelog at https://www.ucunleashed.com/3168 for fixes & changes introduced with each version
	  Wish list             : Better error trapping
	  Rights Required       : N/A
	  Sched Task Required   : No
	  Lync/Skype4B Version  : N/A
	  Author/Copyright      : © Pat Richard, Office Servers and Services (Skype for Business) MVP - All Rights Reserved
	  Email/Blog/Twitter    : pat@innervation.com  https://www.ucunleashed.com  @patrichard
	  Dedicated Post        : https://www.ucunleashed.com/3168
	  Disclaimer            : You running this script means you won't blame me if this breaks your stuff. This script is
	                          provided AS IS without warranty of any kind. I disclaim all implied warranties including,
	                          without limitation, any implied warranties of merchantability or of fitness for a particular
	                          purpose. The entire risk arising out of the use or performance of the sample scripts and
	                          documentation remains with you. In no event shall I be liable for any damages whatsoever
	                          (including, without limitation, damages for loss of business profits, business interruption,
	                          loss of business information, or other pecuniary loss) arising out of the use of or inability
	                          to use the script or documentation.
	  Acknowledgements      : Reading XML files 
	                          http://stackoverflow.com/questions/18509358/how-to-read-xml-in-powershell
	                          http://stackoverflow.com/questions/20433932/determine-xml-node-exists
	  Assumptions           : ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
	  Limitations           : 
	  Known issues          : 

	.LINK
	  https://www.ucunleashed.com/3168

	.EXAMPLE
	  Get-UpdateInfo -Article 123

	  Description
	  -----------
	  Runs function to check for updates to article/script 123.

	.INPUTS
	  None. You cannot pipe objects to this script.
	#>
  [CmdletBinding(SupportsShouldProcess, SupportsPaging)]
  param (
  	[ValidateNotNullorEmpty()]
    [string] $article
  )
  [bool] $HasInternetAccess = ([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]'{DCB00C01-570F-4A9B-8D69-199FDBA5723B}')).IsConnectedToInternet)
  if ($HasInternetAccess){
    [xml] $xml = (New-Object -TypeName System.Net.WebClient).DownloadString('https://www.ucunleashed.com/downloads/version.xml')
    [string] $Ga = ($xml.catalog.article | Where-Object {$_.id -eq $article}).version
    if (($xml.catalog.article | Where-Object {$_.id -eq $article}).versionInfo){
      [string] $changelog = 'This version includes: ' + ($xml.catalog.article | Where-Object {$_.id -eq $article}).versionInfo
    }
    if ($Ga -gt $ScriptVersion){
      Write-Log -Level Warn -Message "Outdated version. Version $Ga is latest version. Prompting user to upgrade." -NoConsole
      if ((New-Popup -Message "Version $Ga is now available. $changelog `n`nWould you like to download it?" -Title 'New version available!' -Buttons YesNo -Icon Question) -eq 6){
        Write-Log -Message 'User elected to download latest version.' -Indent 1 -NoConsole
        Write-Log -Message "Opening Internet Explorer and going to https://www.ucunleashed.com/$article#downloads." -NoConsole -Indent 1
        Start-Process "https://www.ucunleashed.com/$article#downloads"
        Write-Log -Level Warn -Message "Script is exiting. Please start the new version of the script once you've downloaded it." -NoLog
        Exit
      }else{
        Write-Log -Level Warn -Message 'User elected NOT to download latest version.' -Indent 1 -NoConsole
      }
    }elseif ($Ga -eq $ScriptVersion){
      Write-Log -Message "Current version. Version $Ga is latest version." -NoConsole
    }else{
      Write-Log -Level Warn -Message "Possibly regressed version. Version $Ga is latest version." -NoConsole
    }
  }else{
    Write-Log -Level Warn -Message 'Unable to check online for update info.' -NoConsole
  }
} # end function Get-UpdateInfo

function Get-UpdateVersionInfo {
  [CmdletBinding(SupportsShouldProcess)]
  param (
    [parameter(ValueFromPipeline)] 
    [string] $UpdateVersion    
  )
  begin{
    <#
    [bool] $HasInternetAccess = ([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]'{DCB00C01-570F-4A9B-8D69-199FDBA5723B}')).IsConnectedToInternet)
    if ($HasInternetAccess){
      Write-Verbose "Retrieving online version info"
      [xml] $xml = (New-Object System.Net.WebClient).DownloadString("https://www.ucunleashed.com/downloads/2641/version.xml")
      # [xml]$xml = Get-Content "D:\OneDrive\PowerShell\_released\Get-CsUpdateVersion.ps1\version.xml"
    }
    $AllUpdates = @()
    #>
  }
  process{
    if ($xml){
      $UpdateInfo = ($xml.catalog.UpdateVersion | Where-Object {$_.id -eq $UpdateVersion})
      [string] $FriendlyResult = $UpdateInfo.title
      [bool] $UpToDate = [int] $UpdateInfo.up_to_date

      $system = New-Object -TypeName PSObject
      Add-Member -InputObject $system -MemberType NoteProperty -Name Product -Value $($UpdateInfo.Product)
      Add-Member -InputObject $system -MemberType NoteProperty -Name Title -Value $($UpdateInfo.title)
      Add-Member -InputObject $system -MemberType NoteProperty -Name Update -Value $($UpdateInfo.UpdateName)
      Add-Member -InputObject $system -MemberType NoteProperty -Name UpToDate -Value ([bool] ([int] $UpdateInfo.up_to_date))
      Add-Member -InputObject $system -MemberType NoteProperty -Name UpdateUrl -Value $($UpdateInfo.UpdateUrl)
      $AllUpdates += $system 
    }
  }
  end{
    if ($AllUpdates){
      return $AllUpdates
    }
  }
} # end function Get-UpdateVersionInfo

function New-Popup {
  <#
  .Synopsis
    Display a Popup Message

  .Description
    This command uses the Wscript.Shell PopUp method to display a graphical message
    box. You can customize its appearance of icons and buttons. By default the user
    must click a button to dismiss but you can set a timeout value in seconds to
    automatically dismiss the popup.

    The command will write the return value of the clicked button to the pipeline:
      OK     = 1
      Cancel = 2
      Abort  = 3
      Retry  = 4
      Ignore = 5
      Yes    = 6
      No     = 7

    If no button is clicked, the return value is -1.

  .Example
  PS C:\> new-popup -message "The update script has completed" -title "Finished" -time 5

  This will display a popup message using the default OK button and default
  Information icon. The popup will automatically dismiss after 5 seconds.

  .Notes
    Last Updated: April 8, 2013
    Version     : 1.0

  .Inputs
    None

  .Outputs
    integer

    Null   = -1
    OK     = 1
    Cancel = 2
    Abort  = 3
    Retry  = 4
    Ignore = 5
    Yes    = 6
    No     = 7
  #>
  [CmdletBinding(SupportsShouldProcess, SupportsPaging)]
  Param (
    # Message to be displayed in the popup message dialog box.
    [Parameter(Position = 0, Mandatory, HelpMessage = 'Enter a message for the popup')]
    [ValidateNotNullorEmpty()]
    [string] $Message,

    # Title of the popup dialog box.
    [Parameter(Position = 1, Mandatory, HelpMessage = 'Enter a title for the popup')]
    [ValidateNotNullorEmpty()]
    [string] $Title,

    # If defined, automatically dismisses the dialog box after the specified number of seconds. Defaults to 0, which requires a mouse click.
    [Parameter(Position = 2, HelpMessage = 'How many seconds to display? Use 0 require a button click.')]
    [ValidateScript({$_ -ge 0})]
    [int] $Time = 0,

    # Type of buttons to display.
    [Parameter(Position = 3, HelpMessage = 'Enter a button group')]
    [ValidateNotNullorEmpty()]
    [ValidateSet('OK','OKCancel','AbortRetryIgnore','YesNo','YesNoCancel','RetryCancel')]
    [string] $Buttons = 'OK',

    # Icon set to use in dialog box.
    [Parameter(Position = 4, HelpMessage = 'Enter an icon set')]
    [ValidateNotNullorEmpty()]
    [ValidateSet('Stop','Question','Exclamation','Information' )]
    [string] $Icon = 'Information'
  ) # end of param block

  #convert buttons to their integer equivalents
  Switch ($Buttons) {
    'OK'               {$ButtonValue = 0}
    'OKCancel'         {$ButtonValue = 1}
    'AbortRetryIgnore' {$ButtonValue = 2}
    'YesNo'            {$ButtonValue = 4}
    'YesNoCancel'      {$ButtonValue = 3}
    'RetryCancel'      {$ButtonValue = 5}
  }

  #set an integer value for Icon type
  Switch ($Icon) {
    'Stop'        {$iconValue = 16}
    'Question'    {$iconValue = 32}
    'Exclamation' {$iconValue = 48}
    'Information' {$iconValue = 64}
  }

  #create the COM Object
  Try {
    $wshell = New-Object -ComObject Wscript.Shell -ErrorAction Stop
    #Button and icon type values are added together to create an integer value
    $wshell.Popup($Message,$Time,$Title,$ButtonValue+$iconValue)
  }

  Catch {
    #You should never really run into an exception in normal usage
    Write-Log -Level Warn -Message 'Failed to create Wscript.Shell COM object'
    Write-Log -Level Warn -Message $_.exception.message
  }
} # end function New-Popup

function Set-ModuleStatus {
  [CmdletBinding(SupportsShouldProcess)]
  param  (
    [parameter(ValueFromPipeline, ValueFromPipelineByPropertyName, Mandatory, HelpMessage = 'No module name specified!')]
    [ValidateNotNullOrEmpty()]
    [string] $name
  )
  PROCESS{
    # Executes once for each pipeline object
    # the $_ variable represents the current input object
    if (!(Get-Module -name "$name")) {
      if (Get-Module -ListAvailable | Where-Object Name -eq "$name") {
        Import-Module -Name "$name"
        # module was imported
        # return $true
      } else {
        # module was not available
        # return $false
      }
    } else {
      # Write-Output "$_ module already imported"
      # return $true
    }
  } # end PROCESS
} # end function Set-ModuleStatus

function Write-Log {
  <#
  .SYNOPSIS
    Extensive function to write data to either the console screen, a log file, and/or a Windows event log.

  .DESCRIPTION
    Extensive function to write data to either the console screen, a log file, and/or a Windows event log. Data can be written as info, warning, error, and includes indentation, time stamps, etc.

  .NOTES
    Version               : 3.0
    Wish list             : Better error trapping
    Rights Required       : Local administrator on server
    Sched Task Required   : No
    Lync/Skype4B Version  : N/A
    Author/Copyright      : © Pat Richard, Office Servers and Services (Skype for Business) MVP - All Rights Reserved
    Email/Blog/Twitter		: pat@innervation.com 	https://www.ucunleashed.com @patrichard
    Donations             : https://www.paypal.me/PatRichard
    Dedicated Post        :
    Disclaimer            : You running this script means you won't blame me if this breaks your stuff. This script is
                            provided AS IS without warranty of any kind. I disclaim all implied warranties including,
                            without limitation, any implied warranties of merchantability or of fitness for a particular
                            purpose. The entire risk arising out of the use or performance of the sample scripts and
                            documentation remains with you. In no event shall I be liable for any damages whatsoever
                            (including, without limitation, damages for loss of business profits, business interruption,
                            loss of business information, or other pecuniary loss) arising out of the use of or inability
                            to use the script or documentation.
    Acknowledgements      : Test for log names and sources
                            http://powershell.com/cs/blogs/tips/archive/2013/06/10/testing-event-log-names-and-sources.aspx
    Assumptions           : ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
    Limitations           :
    Known issues          :

  .LINK


  .EXAMPLE
    .\

    Description
    -----------


  .INPUTS
    None. You cannot pipe objects to this script.
  #>
  [CmdletBinding(SupportsShouldProcess, SupportsPaging)]
  param(
    # The type of message to be logged. Alias is 'type'.
    [Parameter(Position = 0, ValueFromPipeline, ValueFromPipelineByPropertyName)]
    [ValidateSet('Error', 'Warn', 'Info','Verbose')]
    [ValidateNotNullOrEmpty()]
    [string] $level = 'Info',

    # The message to be logged.
    [Parameter(Position = 1, ValueFromPipeline, ValueFromPipelineByPropertyName, Mandatory, HelpMessage = 'No message specified.')]
    [ValidateNotNullOrEmpty()]
    [string] $Message,

    # Specifies that $message should not the sent to the log file.
    [Parameter(Position = 2, ValueFromPipelineByPropertyName)]
    [switch] $NoLog,

    # Specifies to not display the message to the console.
    [Parameter(Position = 3, ValueFromPipelineByPropertyName)]
    [switch] $NoConsole,

    # The number of spaces to indent the message in the log file.
    [Parameter(Position = 4, ValueFromPipelineByPropertyName)]
    [ValidateRange(1,30)]
    [ValidateNotNullOrEmpty()]
    [Int16] $Indent = 0,

    # Specifies what color the text should be be displayed on the console. Ignored when switch 'NoConsoleOut' is specified.
    [Parameter(Position = 5, ValueFromPipelineByPropertyName)]
    [ValidateSet('Black', 'DarkMagenta', 'DarkRed', 'DarkBlue', 'DarkGreen', 'DarkCyan', 'DarkYellow', 'Red', 'Blue', 'Green', 'Cyan', 'Magenta', 'Yellow', 'DarkGray', 'Gray', 'White')]
    [ValidateNotNullOrEmpty()]
    [String] $ConsoleForeground = 'White',

    # Existing log file is deleted when this is specified. Alias is 'Overwrite'.
    [Parameter(Position = 6, ValueFromPipelineByPropertyName)]
    [Switch] $Clobber,

    # The name of the system event log, e.g. 'Application'. Note that writing to the system event log requires elevated permissions.
    [Parameter(Position = 7, ValueFromPipelineByPropertyName)]
    [ValidateSet('Application','System','Security','Lync Server','Microsoft Office Web Apps')]
    [ValidateNotNullOrEmpty()]
    [String] $EventLogName,

    # The name to appear as the source attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
    [Parameter(Position = 8, ValueFromPipelineByPropertyName)]
    [ValidateNotNullOrEmpty()]
    [String] $EventSource = $($MyInvocation.ScriptName).Name,

    # The ID to appear as the event ID attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
    [Parameter(Position = 9, ValueFromPipelineByPropertyName)]
    [ValidateRange(1,65535)]
    [ValidateNotNullOrEmpty()]
    [Int32] $EventID = 1,

    # The text encoding for the log file. Default is ASCII.
    [Parameter(Position = 10, ValueFromPipelineByPropertyName)]
    [ValidateSet('Unicode','Byte','BigEndianUnicode','UTF8','UTF7','UTF32','ASCII','Default','OEM')]
    [ValidateNotNullOrEmpty()]
    [String] $LogEncoding = 'ASCII'
  ) # end of param block
  try {
    [string]$LogFolder = Split-Path -Path $LogPath -Parent
    if (-not(Test-Path -Path $LogFolder)){$null = New-Item -Path $LogFolder -ItemType Directory}
    $msg = "{0} : {1} : {2}{3}" -f (Get-Date -Format "yyyy-MM-dd HH:mm:ss"), $Level.ToUpper(), ('  ' * $Indent), $Message
    if (-not($NoConsole)){
      switch ($level) {
        'Error' {$Host.UI.WriteErrorLine("$Message")}
        'Warn' {Write-Warning -Message $Message}
        'Info' {Write-Host $Message -ForegroundColor $ConsoleForeground}
        'Verbose' {Write-Verbose -Message $Message}
      }
    }
    if (-not($NoLog)){
      if ($Clobber) {
        $msg | Out-File -FilePath $LogPath -Encoding $LogEncoding -Force
      } else {
        $msg | Out-File -FilePath $LogPath -Encoding $LogEncoding -Append
      }
    }

    # http://social.technet.microsoft.com/Forums/en-US/winserverpowershell/thread/e172f039-ce88-4c9f-b19a-0dd6dc568fa0/

    if ($EventLogName) {
      if (-not $EventSource) {
        [string] $EventSource = $([IO.FileInfo] $MyInvocation.ScriptName).Name
      }

      if(-not [Diagnostics.EventLog]::SourceExists($EventSource)) {
        [Diagnostics.EventLog]::CreateEventSource($EventSource, $EventLogName)
      }

      switch ($Level) {
        'Error' {$EntryType = 'Error'}
        'Warn'  {$EntryType = 'Warning'}
        'Info'  {$EntryType = 'Information'}
        'Verbose' {$EntryType = 'Information'}
        Default  {$EntryType = 'Information'}
      }
      Write-EventLog -LogName $EventLogName -Source $EventSource -EventId 1 -EntryType $EntryType -Message $Message
    }
    $msg = ""
  } # end try
  catch {
    Throw "Failed to create log entry in: '$LogPath'. The error was: '$_'."
  } # end catch
} # end function Write-Log
#endregion functions

#region LogHeader
Write-Log -Level Info -Message "Version         : $ScriptVersion" -NoConsole
Write-Log -Level Info -Message "User            : $env:UserDomain\$env:UserName" -NoConsole
Write-Log -Level Info -Message "Command         : $($MyInvocation.Line)" -NoConsole
Write-Log -Level Info -Message $LogDivider -NoConsole
#endregion LogHeader

Get-UpdateInfo -Article 2641

Set-ModuleStatus -name Lync

[bool] $HasInternetAccess = ([Activator]::CreateInstance([Type]::GetTypeFromCLSID([Guid]'{DCB00C01-570F-4A9B-8D69-199FDBA5723B}')).IsConnectedToInternet)
if ($HasInternetAccess){
  Write-Log -Level Verbose -Message 'Retrieving online version info'
  [xml] $xml = (New-Object -TypeName System.Net.WebClient).DownloadString('https://www.ucunleashed.com/downloads/2641/version.xml')
}
$AllUpdates = @()

if ($PoolFqdn){
  $services = @(Get-CsService | Where-Object {$_.version -ge 5 -and $_.PoolFqdn -eq $PoolFqdn})
}else{
  $services = @(Get-CsService | Where-Object {$_.version -ge 5})
}
$AllowedServices = # @("ApplicationServer","ArchivingServer","BackupServer","CentralManagement","ConferencingServer","Director","EdgeServer","MediationServer","MonitoringServer","PersistentChatServer","Registrar","UserServer","WacServer","WebServer")

@('ApplicationServer','ArchivingServer','BackupServer','CentralManagement','ConferencingServer','Director','EdgeServer','MediationServer','MonitoringServer','PersistentChatServer','Registrar','UserServer','WebServer')

[string] $ComputerFqdn = [Net.Dns]::GetHostByName(($env:ComputerName)).HostName

foreach ($service in $services)  {
  if ($AllowedServices -icontains $service.role) {
    $pools += @($service.PoolFqdn)
  }
}

$pools = $pools | Select-Object -Unique | Sort-Object
Write-Log -Level Verbose -Message "$($pools.count) pools found"
$UpdateObjects = @()
foreach ($Fqdn in $pools)  {
  $computers = @((Get-CsPool $Fqdn).computers | Sort-Object)
  if ($computers) {
    Write-Log -Level Verbose -Message "PoolFqdn: $Fqdn `($($computers.count) servers`)"
    foreach ($computer in $computers) {
      Write-Log -Level Verbose -Message "ComputerName: $computer"
      $UpdateObject = New-Object -TypeName PSObject
      Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name ComputerName -Value $($($computer.split('.')[0]).ToLower().trim())     
      Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name PoolFqdn -Value $($Fqdn.ToLower().trim())          
      if(-not(Test-Connection -ComputerName $computer -BufferSize 16 -Count 1 -TimeToLive $TTL -Quiet -ErrorAction SilentlyContinue)) {
        Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name Version -Value $null
        Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name Platform -Value $null
        Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name UpdateName -Value $null
        Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name FriendlyResult -Value 'Offline'
        Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name UpToDate -Value $null
        $UpdateObjects += $UpdateObject
      } else {
        if ($computer -ne $ComputerFqdn){ # computer is a remote machine
          $version = Invoke-Command -ScriptBlock {
            $filter = @('Debugging Tools','Resource Kit Tools','Best Practices Analyzer','Meeting Room Portal')
            [regex]$regex = '(' + (($filter | foreach {[regex]::escape($_)}) -join '|') + ')'
          @((Get-ChildItem -Path 'HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall' | ForEach-Object {Get-ItemProperty -Path $_.pspath} | Where-Object {($_.DisplayName -imatch 'Microsoft Lync Server' -and $_.DisplayName -inotmatch $regex) -or ($_.DisplayName -imatch 'Microsoft Office Web Apps Server 2013') -or ($_.DisplayName -imatch 'Skype for Business Server' -and $_.DisplayName -inotmatch $regex)}) | Sort-Object -Property DisplayVersion -Descending | Select-Object -Property DisplayVersion,DisplayName -First 1)} -ComputerName $computer -ErrorAction SilentlyContinue -WarningAction SilentlyContinue
          $version = $version.DisplayVersion
        }else{ # computer is a local machine (which sometimes has issues with PSRemoting to/from itself)
          # New regex query to filter out non critical apps
          $filter = @('Debugging Tools','Resource Kit Tools','Best Practices Analyzer','Meeting Room Portal')
          [regex]$regex = '(' + (($filter | foreach {[regex]::escape($_)}) -join '|') + ')'
          $version = (Get-ChildItem -Path 'HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall' | ForEach-Object {Get-ItemProperty -Path $_.pspath} | Where-Object {($_.DisplayName -imatch 'Microsoft Lync Server' -and $_.DisplayName -inotmatch $regex) -or ($_.DisplayName -imatch 'Microsoft Office Web Apps Server 2013') -or ($_.DisplayName -imatch 'Skype for Business Server' -and $_.DisplayName -inotmatch $regex)}) | Sort-Object -Property DisplayVersion -Descending | Select-Object -Property DisplayVersion,DisplayName -First 1
          $version = $version.DisplayVersion
        }
        if (-not($version)) {
          # No version number returned. If we get here, the server responded to ping, so it's no offline. We just couldn't PSRemote to it.
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name Version -Value $null
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name Product -Value $null
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name Update -Value $null
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name FriendlyResult -Value 'PSRemoting failure'
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name UpToDate -Value $null
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name UpdateUrl -Value $null
          $UpdateObjects += $UpdateObject          
        } else {
          # We have valid results
          # Set a default value in case it doesn't match anything below
          $FriendlyResult = 'Unknown Version'
          # [bool] $UpToDate = $false
          $ThisResult = $(Get-UpdateVersionInfo -UpdateVersion $version)
          if ($ThisResult.Title){$FriendlyResult = $ThisResult.Title.trim()}
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name Version -Value $($version.trim())
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name Product -Value $($ThisResult.Product.trim())
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name Update -Value $($ThisResult.Update.trim())
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name FriendlyResult -Value $FriendlyResult
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name UpToDate -Value $($ThisResult.UpToDate)
          Add-Member -InputObject $UpdateObject -MemberType NoteProperty -Name UpdateUrl -Value $($ThisResult.UpdateUrl)
          $UpdateObjects += $UpdateObject
        }
      }
    }
  }else{
    Write-Log -Level Warning -Message 'No computers found'
  }
}
return $UpdateObjects

# SIG # Begin signature block
# MIINJAYJKoZIhvcNAQcCoIINFTCCDRECAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUUPai9eq/N8XCHnKv1nwdHN6F
# sjegggpmMIIFLjCCBBagAwIBAgIQA6cFj2F2VqaerCviGAu9jTANBgkqhkiG9w0B
# AQsFADByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFz
# c3VyZWQgSUQgQ29kZSBTaWduaW5nIENBMB4XDTE2MTAxMjAwMDAwMFoXDTE3MTIy
# MTEyMDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBT
# dGVybGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYD
# VQQDExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
# CgKCAQEAvVNbtzSVPbpRQfYrhU0lPzCnmYKS2DFpgrL8gI8qUsyojdQJRjF8q5YQ
# uIi92Uwl/hxWcvCf0dU/EbzRZudOr8tsbkLA6x1R7O5Fsd6Q2fwMN4YmhFDKUcxf
# Gjy1byOdEhSQZqUc4xp6T+prcIi1j4EDSp5R0TQSRiKUK5HoqanhRyGfqJjkEOep
# KB9soxNVkB+jTXsIjX1AeuUJZS/m+1rzWf75laoid+rna7KQtJO+DkVNSa96tTH2
# 1Fs/C78UzWBLA5I2gQAgEA3alkMzB7rIr34Grt251O+yMSyCcvjb70AVcKohoS7n
# 2Zk522EK3jJw3F8euSWMbXNegJtPjwIDAQABo4IBxTCCAcEwHwYDVR0jBBgwFoAU
# WsS5eyoKo6XqcQPAYPkt9mV1DlgwHQYDVR0OBBYEFE+s6yTjN7ocNskZ2gxbJ3Ul
# xBo6MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzB3BgNVHR8E
# cDBuMDWgM6Axhi9odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vc2hhMi1hc3N1cmVk
# LWNzLWcxLmNybDA1oDOgMYYvaHR0cDovL2NybDQuZGlnaWNlcnQuY29tL3NoYTIt
# YXNzdXJlZC1jcy1nMS5jcmwwTAYDVR0gBEUwQzA3BglghkgBhv1sAwEwKjAoBggr
# BgEFBQcCARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAIBgZngQwBBAEw
# gYQGCCsGAQUFBwEBBHgwdjAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNl
# cnQuY29tME4GCCsGAQUFBzAChkJodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20v
# RGlnaUNlcnRTSEEyQXNzdXJlZElEQ29kZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/
# BAIwADANBgkqhkiG9w0BAQsFAAOCAQEAL3G7W2y46ZvBae5m6LR+rZu1yA1kkmk5
# 55rTY9lA9xCd0jrhY3cPzwKxGFy82surJeWDWbDIao7wqGXIfDrDgFahdM63btdl
# LWJacdfrM7l+82Zx22R30P/K5sVmk1UXr9eyOPzrYMiemUVru3xMNSZ4Md7lHrfb
# CODsEGjs+zWo4eqXWcnnOCxMd1cpr4PvKPonUhMAhCyFDD5TDzZDQIbHBDIvfb8j
# rg/cvZ2PArf5QZpLp5CiSGnGpC+O9rZhYbsOmcJVFMWMM0pMEw46T9HXHxHMfDC1
# oYTp8Xyc/JxiQ+084oNoGng4esFZIb18cJFXDfR0f4ie0pJTMvanqjCCBTAwggQY
# oAMCAQICEAQJGBtf1btmdVNDtW+VUAgwDQYJKoZIhvcNAQELBQAwZTELMAkGA1UE
# BhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2lj
# ZXJ0LmNvbTEkMCIGA1UEAxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4X
# DTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEyMDAwMFowcjELMAkGA1UEBhMCVVMxFTAT
# BgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEx
# MC8GA1UEAxMoRGlnaUNlcnQgU0hBMiBBc3N1cmVkIElEIENvZGUgU2lnbmluZyBD
# QTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAPjTsxx/DhGvZ3cH0wsx
# SRnP0PtFmbE620T1f+Wondsy13Hqdp0FLreP+pJDwKX5idQ3Gde2qvCchqXYJawO
# eSg6funRZ9PG+yknx9N7I5TkkSOWkHeC+aGEI2YSVDNQdLEoJrskacLCUvIUZ4qJ
# RdQtoaPpiCwgla4cSocI3wz14k1gGL6qxLKucDFmM3E+rHCiq85/6XzLkqHlOzEc
# z+ryCuRXu0q16XTmK/5sy350OTYNkO/ktU6kqepqCquE86xnTrXE94zRICUj6whk
# PlKWwfIPEvTFjg/BougsUfdzvL2FsWKDc0GCB+Q4i2pzINAPZHM8np+mM6n9Gd8l
# k9ECAwEAAaOCAc0wggHJMBIGA1UdEwEB/wQIMAYBAf8CAQAwDgYDVR0PAQH/BAQD
# AgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMHkGCCsGAQUFBwEBBG0wazAkBggrBgEF
# BQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMGCCsGAQUFBzAChjdodHRw
# Oi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0Eu
# Y3J0MIGBBgNVHR8EejB4MDqgOKA2hjRodHRwOi8vY3JsNC5kaWdpY2VydC5jb20v
# RGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsMDqgOKA2hjRodHRwOi8vY3JsMy5k
# aWdpY2VydC5jb20vRGlnaUNlcnRBc3N1cmVkSURSb290Q0EuY3JsME8GA1UdIARI
# MEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxodHRwczovL3d3dy5kaWdp
# Y2VydC5jb20vQ1BTMAoGCGCGSAGG/WwDMB0GA1UdDgQWBBRaxLl7KgqjpepxA8Bg
# +S32ZXUOWDAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzANBgkqhkiG
# 9w0BAQsFAAOCAQEAPuwNWiSz8yLRFcgsfCUpdqgdXRwtOhrE7zBh134LYP3DPQ/E
# r4v97yrfIFU3sOH20ZJ1D1G0bqWOWuJeJIFOEKTuP3GOYw4TS63XX0R58zYUBor3
# nEZOXP+QsRsHDpEV+7qvtVHCjSSuJMbHJyqhKSgaOnEoAjwukaPAJRHinBRHoXpo
# aK+bp1wgXNlxsQyPu6j4xRJon89Ay0BEpRPw5mQMJQhCMrI2iiQC/i9yfhzXSUWW
# 6Fkd6fp0ZGuy62ZD2rOwjNXpDd32ASDOmTFjPQgaGLOBm0/GkxAG/AeB+ova+YJJ
# 92JuoVP6EpQYhS6SkepobEQysmah5xikmmRR7zGCAigwggIkAgEBMIGGMHIxCzAJ
# BgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5k
# aWdpY2VydC5jb20xMTAvBgNVBAMTKERpZ2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBD
# b2RlIFNpZ25pbmcgQ0ECEAOnBY9hdlamnqwr4hgLvY0wCQYFKw4DAhoFAKB4MBgG
# CisGAQQBgjcCAQwxCjAIoAKAAKECgAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcC
# AQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwIwYJKoZIhvcNAQkEMRYE
# FOtEQC15UtGI0xUwsvTs7I8+1pm9MA0GCSqGSIb3DQEBAQUABIIBAD4+9ktLuUJO
# J8nH5i0/BaT7YEuB+onJjFHekVqxuThCWdGKNBs84LJ7Nk5wUdDuzXICEQ+u9uSw
# yzlshObO4xADxN7rUYxRIea6NrMmQPDNsoNbfgZd1JS9zXzmKR4oau+MKA1PwqyX
# qBP4dj7DkTTlTzQIe8hTGivt9C6uDx0U0k25pFoGQ8j0MAcHplYz5rTcfpdJFIk8
# j85lbt52tmZr6OcwuHaWeTcHowVc59LhIbtW6hE7TOU6PoW/vWD6fK0MgskV7lk2
# 0XV10h+ffgFKpipn6ec0qeDPYCz/6TGdUpEcILFUMmede9rQgfpLa8xLX4J6NXud
# EnFmzR6NUlI=
# SIG # End signature block
